#!/usr/bin/env python
"""
Scummbler v2

    This file is part of Scummbler.

    Scummbler is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Scummbler is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Scummbler.  If not, see <http://www.gnu.org/licenses/>.

Laurence Dougal Myers
www.jestarjokin.net

December 2008 - May 2009

Atomic parsing elements
"""
from pyparsing import *
import scummbler_vars

# le = lexicon entry, also gives the code a French feel

# Numbers
leUnsignedInt = Word(nums)
leInt = Word(nums + "-", nums)
leHexByte = Word(nums + srange("[a-fA-F]"), exact=2) # for descummed scripts
leHexWord = Word(nums + srange("[a-fA-F]"), exact=4) # for descummed scripts
leDefinedValueDef = Word(alphanums + "_") # prevents parsing on definition
leDefinedValue = Word("$", alphanums + "_") # compiler define value (I hate using the $)
leByte = (leInt | leDefinedValue)("value")
leWord = (leInt | leDefinedValue)("value")
leDelay = leUnsignedInt
leLabel = Word(alphanums + "_")
leLocalIndex = leUnsignedInt
leBitIndex = leUnsignedInt # same for now
leGlobalIndex = leUnsignedInt # same for now
leLocalIndexOrPointer = Forward() # I'm not sure if this can be indirect
leBitIndexOrPointer = Forward() # I'm not sure if this can be indirect
leGlobalIndexOrPointer = Forward()

# Variables (these could probably be condensed)
leLocalVar = Literal("Local")("vartype") + Suppress(Literal("[")) + leLocalIndexOrPointer("value") + Suppress(Literal("]"))
leBitVar = Literal("Bit")("vartype") + Suppress(Literal("[")) + leBitIndexOrPointer("value") + Suppress(Literal("]"))
leGlobalVar = Literal("Var")("vartype") + Suppress(Literal("[")) + leGlobalIndexOrPointer("value") + Suppress(Literal("]"))
leKnownGlobalVar = oneOf(" ".join( (vn for vn in scummbler_vars.varNames5list if vn != None) ))("knownvar")

leVariable = leLocalVar | leBitVar | leGlobalVar | leKnownGlobalVar | leDefinedValue
leVarOrByte = leVariable | leByte
leVarOrWord = leVariable | leWord
leListItem = Group(leVariable | leWord) # group is a hack to work around nested vars etc

leIndexPointer = Group(leUnsignedInt("base") + Literal("+") + Group(leVarOrWord)("offset"))

# Numbers (Forwarded/Recursive)
leLocalIndexOrPointer << (leIndexPointer | leLocalIndex) # removed because only globals can be indirect
leBitIndexOrPointer << (leIndexPointer | leBitIndex) # removed because only globals can be indirect
leGlobalIndexOrPointer << (leIndexPointer | leGlobalIndex)

leArgList = Suppress(Literal("[")) + Optional(delimitedList(leListItem))("list") + Suppress(Literal("]"))

# Other
leString = quotedString # this has its own parse action, due to descumm's poor handling of escape chars
leComment = cStyleComment # luckily, descumm uses C-style comments!
